@@ -2,10 +2,13 @@
 use elbus::client::AsyncClient;
 use elbus::rpc::{Rpc, RpcClient, RpcError, RpcEvent, RpcResult};
 use elbus::QoS;
-use eva_common::events::{LOG_INPUT_TOPIC, SERVICE_STATUS_TOPIC};
+use eva_common::events::{
+    ANY_STATE_TOPIC, LOCAL_STATE_TOPIC, LOG_INPUT_TOPIC, REMOTE_STATE_TOPIC, SERVICE_STATUS_TOPIC,
+};
 use eva_common::prelude::*;
 use eva_common::services;
 use eva_common::SLEEP_STEP;
+use lazy_static::lazy_static;
 use log::{Level, Log};
 use std::collections::HashMap;
 use std::io::Read;
@@ -17,6 +20,61 @@ use tokio::io::AsyncReadExt;
 use tokio::signal::unix::{signal, SignalKind};
 use tokio::time::sleep;
 
+#[derive(Debug, Copy, Clone)]
+pub enum EventKind {
+    Local,
+    Remote,
+    Any,
+}
+
+impl EventKind {
+    #[inline]
+    pub fn topic(&self) -> &str {
+        match self {
+            EventKind::Local => LOCAL_STATE_TOPIC,
+            EventKind::Remote => REMOTE_STATE_TOPIC,
+            EventKind::Any => ANY_STATE_TOPIC,
+        }
+    }
+}
+
+pub async fn subscribe_oids<R: Rpc>(rpc: &R, oids: &[OID], kind: EventKind) -> EResult<()> {
+    let topics: Vec<String> = oids
+        .iter()
+        .map(|oid| format!("{}{}", kind.topic(), oid.to_path()))
+        .collect();
+    rpc.client()
+        .lock()
+        .await
+        .subscribe_bulk(topics.iter().map(String::as_str).collect(), QoS::No)
+        .await?;
+    Ok(())
+}
+
+lazy_static! {
+    static ref NEED_PANIC: Mutex<Option<Duration>> = <_>::default();
+}
+
+/// # Panics
+///
+/// Will panic if the mutex is poisoned
+pub fn set_poc(panic_in: Option<Duration>) {
+    *NEED_PANIC.lock().unwrap() = panic_in;
+}
+
+/// # Panics
+///
+/// Will panic if the mutex is poisoned
+pub fn poc() {
+    if let Some(delay) = *NEED_PANIC.lock().unwrap() {
+        bmart::process::suicide(delay + Duration::from_secs(1), false);
+        std::thread::spawn(move || {
+            std::thread::sleep(delay);
+            std::process::exit(1);
+        });
+    }
+}
+
 pub fn handle_default_rpc(method: &str, info: &services::ServiceInfo) -> RpcResult {
     match method {
         "test" => Ok(None),
